@@ -55,6 +55,108 @@ typedef struct {
 
 GtkWidget *bah_window = NULL;
 
+/*
+ * set_access_name_desc
+ * Description : Set Accessible Name and Description.
+ */
+static void
+set_access_name_desc (GtkWidget *widget, Fish *fish)
+{
+	AtkObject *obj;
+	static GtkWidget *w = NULL;
+	gchar *fish_name, *acc_desc, *acc_name;
+
+	const gchar *name_format = _("%s the GNOME Fish");
+	const gchar *desc_format = _("%s the GNOME Fish, a contemporary oracle");
+
+	fish_name = panel_applet_gconf_get_string (PANEL_APPLET (fish->applet),
+						   FISH_PREFS_NAME,
+						   NULL);
+
+	if (widget)
+		w = widget;
+
+	if (w) 
+		obj = gtk_widget_get_accessible (GTK_WIDGET (w));
+	else
+		return;
+
+	/* Return immediately if GAIL is not loaded */
+	if (!GTK_IS_ACCESSIBLE (obj))
+		return;
+
+	acc_name = g_strdup_printf (name_format, fish_name);
+	atk_object_set_name (obj, acc_name);
+	g_free (acc_name);       
+
+	acc_desc = g_strdup_printf (desc_format, fish_name);
+	atk_object_set_description (obj, acc_desc);
+	g_free (acc_desc);
+}
+
+static void
+set_tooltip (GtkWidget *applet)
+{
+	GtkTooltips *tooltips;
+
+	tooltips = gtk_tooltips_new ();
+	g_object_ref (tooltips);
+	gtk_object_sink (GTK_OBJECT (tooltips));
+
+	g_object_set_data (G_OBJECT (applet), "tooltips", tooltips);
+
+	gtk_tooltips_set_tip (tooltips, applet,
+			      _("GNOME Fish, the fortune teller"), NULL);
+}
+
+static void
+destroy_tooltip (GtkWidget *applet)
+{
+	GtkTooltips *tooltips;
+
+	tooltips = g_object_get_data (G_OBJECT (applet), "tooltips");
+	if (tooltips) {
+		g_object_unref (tooltips);
+		g_object_set_data (G_OBJECT (applet), "tooltips", NULL);
+	}
+}
+
+/*
+ * set_relation
+ * @widget : The Gtk widget whose corresponding label is @label
+ * @label : The label for the @widget.
+ * Description : This function establishes the labelled-by/for relation
+ * between any Gtk widget and a Gtk label.
+ */
+static void
+set_relation (GtkWidget *widget, GtkLabel *label)
+{
+	AtkObject      *aobject;
+	AtkRelationSet *relation_set;
+	AtkRelation    *relation;
+	AtkObject      *targets [1];
+
+	g_return_if_fail (GTK_IS_WIDGET(widget));
+	g_return_if_fail (GTK_IS_LABEL(label));
+
+	aobject = gtk_widget_get_accessible (widget);
+
+	/* Set the ATK_RELATION_LABEL_FOR relation */
+	gtk_label_set_mnemonic_widget (label, widget);
+
+	/* return if gail is not loaded */
+	if ( ! GTK_IS_ACCESSIBLE (aobject))
+		return;
+
+	targets [0] = gtk_widget_get_accessible (GTK_WIDGET (label));
+
+	relation_set = atk_object_ref_relation_set (aobject);
+
+	relation = atk_relation_new (targets, 1, ATK_RELATION_LABELLED_BY);
+	atk_relation_set_add (relation_set, relation);
+	g_object_unref (relation);
+}
+
 static gboolean
 fish_applet_rotate (Fish *fish)
 {
@@ -137,7 +239,7 @@ set_wanda_day(void)
 static void
 load_image_file (Fish *fish)
 {
-	GdkPixbuf *pixbuf;
+	GdkPixbuf *pixbuf = NULL;
 	GError    *error = NULL;
 	double     affine [6];
 	guchar    *rgb;
@@ -281,17 +383,19 @@ setup_size (Fish *fish)
 	frames = panel_applet_gconf_get_int (PANEL_APPLET (fish->applet), FISH_PREFS_FRAMES, NULL);
 
 	if (fish_applet_rotate (fish)) {
-		gtk_drawing_area_size (GTK_DRAWING_AREA (fish->darea),
-				       fish->w,
-				       fish->h / frames);
+		GTK_WIDGET (fish->darea)->requisition.width = fish->w;
+		GTK_WIDGET (fish->darea)->requisition.height =
+							fish->h / frames;
+		gtk_widget_queue_resize (GTK_WIDGET (fish->darea));
 
 		gtk_widget_set_usize (fish->darea,
 				      fish->w,
 				      fish->h / frames);
 	} else {
-		gtk_drawing_area_size (GTK_DRAWING_AREA (fish->darea), 
-				       fish->w / frames,
-				       fish->h);
+		GTK_WIDGET (fish->darea)->requisition.width =
+							fish->w / frames;
+		GTK_WIDGET (fish->darea)->requisition.height = fish->h;
+		gtk_widget_queue_resize (GTK_WIDGET (fish->darea));
 
 		gtk_widget_set_usize (fish->darea,
 				      fish->w / frames,
@@ -303,17 +407,6 @@ setup_size (Fish *fish)
 	gtk_widget_queue_resize (fish->darea->parent->parent);
 }
 
-static char *
-splice_name(const char * format, const char * name)
-{
-	char * buf;
-	int len;
-	len = strlen(name) + strlen(format);
-	buf = g_malloc(len+1);
-	g_snprintf(buf, len, format, name);
-	return buf;
-}
-
 static void
 fish_draw (GtkWidget *darea,
 	   Fish      *fish)
@@ -389,8 +482,8 @@ apply_dialog_properties (Fish *fish)
 {
 	/* xgettext:no-c-format */
 	const char *title_format = _("%s the Fish");
-	const char *label_format = _("%s the GNOME Fish Says:");
-	char *name, *tmp, *fmt;
+	const char *label_format = _("<big><big>%s the GNOME Fish Says:</big></big>");
+	char *name, *tmp;
 
 	if (fish->fortune_dialog == NULL)
 		return;
@@ -399,13 +492,11 @@ apply_dialog_properties (Fish *fish)
 					      FISH_PREFS_NAME,
 					      NULL);
 
-	tmp = splice_name (title_format, name);
+	tmp = g_strdup_printf (title_format, name);
 	gtk_window_set_title (GTK_WINDOW (fish->fortune_dialog), tmp);
 	g_free (tmp);
 
-	fmt = g_strdup_printf ("<big><big>%s</big></big>", label_format);
-	tmp = splice_name (fmt, name);
-	g_free (fmt);
+	tmp = g_strdup_printf (label_format, name);
 	gtk_label_set_markup (GTK_LABEL (fish->fortune_label), tmp);
 	g_free (tmp);
 
@@ -435,9 +526,7 @@ apply_properties (Fish *fish)
 }
 
 static void
-fish_properties_apply_callback (GnomePropertyBox *pb,
-				int               page,
-				Fish             *fish)
+fish_properties_apply (GtkDialog *pb, Fish *fish)
 {
 	GtkWidget     *name;
 	GtkWidget     *image;
@@ -455,9 +544,6 @@ fish_properties_apply_callback (GnomePropertyBox *pb,
 	rotate  = gtk_object_get_data (GTK_OBJECT (pb), "rotate");
 	command = gtk_object_get_data (GTK_OBJECT (pb), "command");
 
-	if (page != -1) 
-		return;
-
 	text = gtk_entry_get_text (GTK_ENTRY (name));
 	if (text)
 		panel_applet_gconf_set_string (PANEL_APPLET (fish->applet),
@@ -522,11 +608,13 @@ fish_properties_apply_callback (GnomePropertyBox *pb,
 				     gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (rotate)),
 				     NULL);
 
+	set_access_name_desc (NULL,fish); 
+
 	apply_properties (fish);
 }
 
 static void
-phelp_cb (GtkWidget *w, gint tab, gpointer data)
+phelp (void)
 {
 	GError *error = NULL;
 
@@ -537,6 +625,38 @@ phelp_cb (GtkWidget *w, gint tab, gpointer data)
 	}
 }
 
+static void
+changed_cb (GtkWidget *button, gpointer data)
+{
+	gtk_widget_set_sensitive (GTK_WIDGET(button), TRUE);
+}
+
+static void
+response_cb (GtkDialog *dialog, gint id, gpointer data) 
+{
+	Fish *fish = data;
+
+	switch (id) 
+	{
+		case GTK_RESPONSE_OK:
+			fish_properties_apply (dialog, fish);
+			gtk_widget_destroy (GTK_WIDGET(dialog));
+			fish->pb = NULL;
+			break;
+		case GTK_RESPONSE_APPLY:
+			fish_properties_apply (dialog, fish);
+			break;
+		case GTK_RESPONSE_HELP:
+			phelp ();
+			break;
+		case GTK_RESPONSE_CLOSE:
+			gtk_widget_destroy (GTK_WIDGET(dialog));
+			fish->pb = NULL;
+			break;
+	}
+}
+
+
 static void 
 display_properties_dialog (BonoboUIComponent *uic,
 			   Fish              *fish,
@@ -546,6 +666,7 @@ display_properties_dialog (BonoboUIComponent *uic,
 	GtkWidget     *hbox;
 	GtkWidget     *w;
 	GtkWidget     *e;
+	GtkWidget     *l;
 	GtkAdjustment *adj;
 	PanelApplet   *applet;
 	gchar         *name;
@@ -554,6 +675,7 @@ display_properties_dialog (BonoboUIComponent *uic,
 	gint           frames;
 	gdouble        speed;
 	gboolean       rotate;
+	GtkWidget     *apply_button;
 
 	if (fish->pb != NULL) {
 		gtk_window_present (GTK_WINDOW (fish->pb));
@@ -569,10 +691,21 @@ display_properties_dialog (BonoboUIComponent *uic,
 	speed   = panel_applet_gconf_get_float  (applet, FISH_PREFS_SPEED, NULL);
 	rotate  = panel_applet_gconf_get_bool   (applet, FISH_PREFS_ROTATE, NULL);
 
-	fish->pb = gnome_property_box_new ();
+	fish->pb = gtk_dialog_new ();
+	gtk_dialog_add_button (GTK_DIALOG (fish->pb), 
+				GTK_STOCK_HELP, GTK_RESPONSE_HELP);
+	apply_button = gtk_dialog_add_button (GTK_DIALOG (fish->pb), 
+				GTK_STOCK_APPLY, GTK_RESPONSE_APPLY);
+	gtk_widget_set_sensitive (GTK_WIDGET(apply_button), FALSE);
+	gtk_dialog_add_buttons (GTK_DIALOG (fish->pb), 
+				GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, 
+				GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
 	gtk_window_set_wmclass (GTK_WINDOW (fish->pb), "fish", "Fish");
+
+	gtk_dialog_set_default_response (GTK_DIALOG (fish->pb), GTK_RESPONSE_OK);
+
 	gtk_window_set_title (GTK_WINDOW (fish->pb),
-			      _("GNOME Fish Properties"));
+			      _("GNOME Fish Preferences"));
 	gnome_window_icon_set_from_file (GTK_WINDOW (fish->pb),
 					 GNOME_ICONDIR "/gnome-fish.png");
 
@@ -582,99 +715,103 @@ display_properties_dialog (BonoboUIComponent *uic,
 	hbox = gtk_hbox_new (FALSE, GNOME_PAD);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
-	w = gtk_label_new (_("Your GNOME Fish's Name:"));
+	w = gtk_label_new_with_mnemonic (_("_Name of GNOME fish:"));
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	e = gtk_entry_new ();
 	gtk_entry_set_text (GTK_ENTRY(e), name);
 	gtk_box_pack_start (GTK_BOX (hbox), e, TRUE, TRUE, 0);
 	gtk_object_set_data (GTK_OBJECT (fish->pb),"name",e);
 
+	set_relation (e, GTK_LABEL (w));
+
 	gtk_signal_connect_object_while_alive (GTK_OBJECT (e),
 					       "changed",
-					       GTK_SIGNAL_FUNC (gnome_property_box_changed),
-					       GTK_OBJECT (fish->pb));
+					       GTK_SIGNAL_FUNC (changed_cb),
+					       GTK_OBJECT (apply_button));
 
 	hbox = gtk_hbox_new (FALSE, GNOME_PAD);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
-	w = gtk_label_new (_("The Animation Filename:"));
-	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
+	l = gtk_label_new_with_mnemonic (_("_Animation filename:"));
+	gtk_box_pack_start (GTK_BOX (hbox), l, FALSE, FALSE, 0);
 	w = gnome_pixmap_entry_new ("fish_animation", _("Browse"), TRUE);
 	gtk_box_pack_start (GTK_BOX (hbox), w, TRUE, TRUE, 0);
 	e = gnome_pixmap_entry_gtk_entry (GNOME_PIXMAP_ENTRY (w));
 	gtk_entry_set_text (GTK_ENTRY (e), image);
 	gtk_object_set_data (GTK_OBJECT (fish->pb), "image", w);
 
+	set_relation (e, GTK_LABEL (l));
+
 	gtk_signal_connect_object_while_alive (GTK_OBJECT (e),
 					       "changed",
-					       GTK_SIGNAL_FUNC (gnome_property_box_changed),
-					       GTK_OBJECT (fish->pb));
+					       GTK_SIGNAL_FUNC (changed_cb),
+					       GTK_OBJECT (apply_button));
 
 	hbox = gtk_hbox_new (FALSE, GNOME_PAD);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
-	w = gtk_label_new (_("Command to execute when fish is clicked:"));
+	w = gtk_label_new_with_mnemonic (_("Command to _execute when fish is clicked:"));
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	e = gtk_entry_new ();
 	gtk_entry_set_text (GTK_ENTRY (e), command);
 	gtk_box_pack_start (GTK_BOX (hbox), e, TRUE, TRUE, 0);
 	gtk_object_set_data (GTK_OBJECT (fish->pb), "command", e);
 
+	set_relation (e, GTK_LABEL (w));
+
 	gtk_signal_connect_object_while_alive (GTK_OBJECT (e),
 					       "changed",
-					       GTK_SIGNAL_FUNC (gnome_property_box_changed),
-					       GTK_OBJECT (fish->pb));
+					       GTK_SIGNAL_FUNC (changed_cb),
+					       GTK_OBJECT (apply_button));
 
 	hbox = gtk_hbox_new (FALSE, GNOME_PAD);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
-	w = gtk_label_new (_("Frames In Animation:"));
+	w = gtk_label_new_with_mnemonic (_("Frame_s in animation:"));
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	adj = (GtkAdjustment *) gtk_adjustment_new (frames, 1.0, 255.0, 1.0, 5.0, 0.0);
-	w = gtk_spin_button_new (adj, 0, 0);
-	gtk_widget_set_usize (w, 70, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
+	e = gtk_spin_button_new (adj, 0, 0);
+	gtk_widget_set_usize (e, 70, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), e, FALSE, FALSE, 0);
 	gtk_object_set_data (GTK_OBJECT (fish->pb), "frames", adj);
 
+	set_relation (e, GTK_LABEL (w));
+
 	gtk_signal_connect_object (GTK_OBJECT (adj),
 				   "value_changed",
-				   GTK_SIGNAL_FUNC (gnome_property_box_changed),
-				   GTK_OBJECT (fish->pb));
+				   GTK_SIGNAL_FUNC (changed_cb),
+				   GTK_OBJECT (apply_button));
 
 	hbox = gtk_hbox_new (FALSE, GNOME_PAD);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
-	w = gtk_label_new (_("Pause per frame (s):"));
+	w = gtk_label_new_with_mnemonic (_("Pause _per frame (s):"));
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	adj = (GtkAdjustment *) gtk_adjustment_new (speed, 0.1, 10.0, 0.1, 1.0, 0.0);
-	w = gtk_spin_button_new (adj, 0, 2);
-	gtk_widget_set_usize (w, 70, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
+	e = gtk_spin_button_new (adj, 0, 2);
+	gtk_widget_set_usize (e, 70, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), e, FALSE, FALSE, 0);
 	gtk_object_set_data (GTK_OBJECT (fish->pb), "speed", adj);
 
+	set_relation (e, GTK_LABEL (w));
+
 	gtk_signal_connect_object (GTK_OBJECT (adj),
 				   "value_changed",
-				   GTK_SIGNAL_FUNC (gnome_property_box_changed),
-				   GTK_OBJECT (fish->pb));
+				   GTK_SIGNAL_FUNC (changed_cb),
+				   GTK_OBJECT (apply_button));
 
-	w = gtk_check_button_new_with_label (_("Rotate on vertical panels"));
+	w = gtk_check_button_new_with_mnemonic (_("_Rotate on vertical panels"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w), rotate);
 	gtk_signal_connect_object (GTK_OBJECT (w),
 				   "toggled",
-				   GTK_SIGNAL_FUNC (gnome_property_box_changed),
-				   GTK_OBJECT (fish->pb));
+				   GTK_SIGNAL_FUNC (changed_cb),
+				   GTK_OBJECT (apply_button));
 
 	gtk_object_set_data (GTK_OBJECT (fish->pb), "rotate", w);
 	gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 0);
 
-	gnome_property_box_append_page (GNOME_PROPERTY_BOX (fish->pb),
-					vbox,
-					gtk_label_new (_("Fish")));
-
-	gtk_signal_connect (GTK_OBJECT(fish->pb),
-			    "apply",
-			    GTK_SIGNAL_FUNC (fish_properties_apply_callback),
-			    fish);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (fish->pb)->vbox), 
+	                    vbox, TRUE, TRUE, 0);
 
 	gtk_signal_connect (GTK_OBJECT(fish->pb),
 			    "destroy",
@@ -682,9 +819,9 @@ display_properties_dialog (BonoboUIComponent *uic,
 			    &fish->pb);
 
 	gtk_signal_connect (GTK_OBJECT (fish->pb),
-			    "help",
-			    GTK_SIGNAL_FUNC (phelp_cb),
-			    NULL);
+			    "response",
+			    GTK_SIGNAL_FUNC (response_cb),
+			    fish);
 
 	gtk_widget_show_all (fish->pb);
 
@@ -723,9 +860,8 @@ insert_text (Fish *fish, const char *text)
 
 	gtk_text_buffer_get_iter_at_offset (fish->fortune_buffer, &iter, -1);
 
-	gtk_text_buffer_insert_with_tags_by_name (fish->fortune_buffer, &iter,
-						  text, -1,
-						  "monospace", NULL);
+	gtk_text_buffer_insert (fish->fortune_buffer, &iter,
+				text, -1);
 }
 
 static void
@@ -760,7 +896,7 @@ static void
 update_fortune_dialog (Fish *fish)
 {
 	char *fortune_command;
-	FILE *fp;
+	gchar *output = NULL;
 
 	if ( fish->fortune_dialog == NULL ) {
 		GtkWidget *view;
@@ -773,6 +909,9 @@ update_fortune_dialog (Fish *fish)
 						     GTK_STOCK_CLOSE,
 						     GTK_RESPONSE_CLOSE,
 						     NULL);
+
+		gtk_dialog_set_default_response (GTK_DIALOG (fish->fortune_dialog), GTK_RESPONSE_CLOSE);
+
 		g_signal_connect (G_OBJECT (fish->fortune_dialog), "delete_event",
 				  G_CALLBACK (delete_event), NULL);
 		g_signal_connect (G_OBJECT (fish->fortune_dialog), "response",
@@ -793,11 +932,6 @@ update_fortune_dialog (Fish *fish)
 		fish->fortune_buffer =
 			gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
 
-		gtk_text_buffer_create_tag (fish->fortune_buffer, "monospace",
-					    "family", "monospace",
-					    NULL);
-
-
 		sw = gtk_scrolled_window_new (NULL, NULL);
 		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
 						GTK_POLICY_AUTOMATIC,
@@ -807,13 +941,15 @@ update_fortune_dialog (Fish *fish)
 
 		gtk_container_add (GTK_CONTAINER (sw), view);
 
+		set_access_name_desc (view, fish);
+
 		fish->fortune_label = gtk_label_new ("");
 
-		gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (fish->fortune_dialog)->vbox), 
+		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (fish->fortune_dialog)->vbox), 
 				    fish->fortune_label,
 				    FALSE, FALSE, GNOME_PAD);
 
-		gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (fish->fortune_dialog)->vbox), 
+		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (fish->fortune_dialog)->vbox), 
 				    sw,
 				    TRUE, TRUE, GNOME_PAD);
 
@@ -821,25 +957,20 @@ update_fortune_dialog (Fish *fish)
 
 		gtk_widget_show_all (fish->fortune_dialog);
 	} else {
+		set_access_name_desc (NULL, fish);
 		gtk_window_present (GTK_WINDOW (fish->fortune_dialog));
 	}
 
 	text_clear (fish);
 
-	fp = NULL;
-
 	fortune_command = fish_locate_fortune_command (fish);
-	if (fortune_command != NULL) {
-		fp = popen (fortune_command, "r");
-	}
+
+	g_spawn_command_line_sync (fortune_command, &output, NULL, NULL, NULL);
+	
 	g_free (fortune_command);
 
-	if (fp != NULL) {
-		char buf[2048];
-		while (fgets (buf, 2048, fp) != NULL) {
-			insert_text (fish, buf);
-		}
-		pclose (fp);
+	if (output) {
+		insert_text (fish, output);
 	} else {
 		insert_text (fish, 
 			     _("You do not have fortune installed "
@@ -847,6 +978,24 @@ update_fortune_dialog (Fish *fish)
 			       "to run.\n\nPlease refer to fish "
 			       "properties dialog."));
 	}
+
+}
+
+static void
+change_water (Fish *fish)
+{
+	GtkWidget *w;
+	w = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_INFO,
+				    GTK_BUTTONS_OK,
+				    _("The water needs changing!\n"
+				    "(Look at today's date)"));
+	gtk_window_set_wmclass (GTK_WINDOW (w), "fish", "Fish");
+
+	gtk_widget_show_all (w);
+
+	g_signal_connect_swapped (G_OBJECT (w), "response",
+				  G_CALLBACK (gtk_widget_destroy),
+				  G_OBJECT (w));
 }
 
 static gboolean 
@@ -859,18 +1008,7 @@ fish_clicked_cb (GtkWidget * widget, GdkEventButton * e, Fish *fish)
 
 	/* on 1st of april the fish is dead damnit */
 	if (fish->april_fools) {
-		GtkWidget *w;
-		w = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_INFO,
-					    GTK_BUTTONS_OK,
-					    _("The water needs changing!\n"
-					      "(Look at today's date)"));
-		gtk_window_set_wmclass (GTK_WINDOW (w), "fish", "Fish");
-
-		gtk_widget_show_all (w);
-
-		g_signal_connect_swapped (G_OBJECT (w), "response",
-					  G_CALLBACK (gtk_widget_destroy),
-					  G_OBJECT (w));
+		change_water (fish);
 		return TRUE;
 	}
 
@@ -879,6 +1017,33 @@ fish_clicked_cb (GtkWidget * widget, GdkEventButton * e, Fish *fish)
 	return TRUE; 
 }
 
+static gboolean
+fish_keypress_cb (GtkWidget * widget, GdkEventKey * e, Fish *fish)
+{
+	/* Invoke fortune dialog when Enter or Space key is pressed */
+	switch (e->keyval)
+	{
+		case GDK_space:
+		case GDK_KP_Space:
+		case GDK_Return:
+		case GDK_KP_Enter:
+		case GDK_ISO_Enter:
+		case GDK_3270_Enter:
+			if (fish->april_fools) {
+				change_water (fish);
+				return TRUE;
+			}
+
+			update_fortune_dialog(fish);
+			break;
+		default:
+			/* Other keys are not handled here */
+			return FALSE;
+	}
+
+	return TRUE;
+}
+
 static int
 fish_expose (GtkWidget      *darea,
 	     GdkEventExpose *event,
@@ -921,14 +1086,24 @@ create_fish_widget(Fish *fish)
 
 	fish->darea = gtk_drawing_area_new();
 
-	if (fish_applet_rotate (fish))
-		gtk_drawing_area_size (GTK_DRAWING_AREA (fish->darea),
-				       fish->w,
-				       fish->h / frames);
-	else
-		gtk_drawing_area_size (GTK_DRAWING_AREA (fish->darea),
-				       fish->w / frames,
-				       fish->h);
+	if (fish_applet_rotate (fish)) {
+		GTK_WIDGET (fish->darea)->requisition.width = fish->w;
+		GTK_WIDGET (fish->darea)->requisition.height =
+							fish->h / frames;
+		gtk_widget_queue_resize (GTK_WIDGET (fish->darea));
+		gtk_widget_set_usize (fish->darea,
+				      fish->w,
+				      fish->h / frames);
+	} else {
+		GTK_WIDGET (fish->darea)->requisition.width =
+							fish->h / frames;
+		GTK_WIDGET (fish->darea)->requisition.width = fish->w;
+
+		gtk_widget_queue_resize (GTK_WIDGET (fish->darea));
+		gtk_widget_set_usize (fish->darea,
+				      fish->w / frames,
+				      fish->h);
+	}
 
 	gtk_widget_set_events(fish->darea, gtk_widget_get_events(fish->darea) |
 			      GDK_BUTTON_PRESS_MASK);
@@ -945,8 +1120,8 @@ create_fish_widget(Fish *fish)
         fish->timeout_id = gtk_timeout_add (speed * 1000, fish_timeout, fish);
 
         gtk_container_add(GTK_CONTAINER(fish->frame),fish->darea);
-
-	gtk_widget_pop_colormap ();
+        
+        gtk_widget_pop_colormap ();
 }
 
 static void
@@ -985,7 +1160,7 @@ display_about_dialog (BonoboUIComponent *uic,
 
 	name = panel_applet_gconf_get_string (PANEL_APPLET (fish->applet), FISH_PREFS_NAME, NULL);
 
-	authors[0] = splice_name (author_format, name);
+	authors[0] = g_strdup_printf (author_format, name);
 	authors[1] = _("(with minor help from George)");
 	authors[2] = NULL;
 
@@ -1004,7 +1179,7 @@ display_about_dialog (BonoboUIComponent *uic,
 	}
 		
 	fish->aboutbox =
-		gnome_about_new (_("The GNOME Fish Applet"),
+		gnome_about_new (_("Fish"),
 				 "3.4.7.4ac19",
 				 "(C) 1998 the Free Software Foundation",
 				 _("This applet has no use what-so-ever. "
@@ -1036,6 +1211,8 @@ display_about_dialog (BonoboUIComponent *uic,
 static void
 applet_destroy (GtkWidget *applet, Fish *fish)
 {
+	destroy_tooltip (GTK_WIDGET (fish->applet));
+
 	if (fish->pix != NULL)
 		gdk_pixmap_unref(fish->pix);
 	fish->pix = NULL;
@@ -1102,13 +1279,15 @@ applet_change_size (PanelApplet *applet,
 }
 
 static const BonoboUIVerb fish_menu_verbs [] = {
-	BONOBO_UI_UNSAFE_VERB ("FishProperties", display_properties_dialog),
-	BONOBO_UI_UNSAFE_VERB ("FishHelp",       display_help_dialog),
-	BONOBO_UI_UNSAFE_VERB ("FishAbout",      display_about_dialog),
+	BONOBO_UI_UNSAFE_VERB ("FishPreferences", display_properties_dialog),
+	BONOBO_UI_UNSAFE_VERB ("FishHelp",        display_help_dialog),
+	BONOBO_UI_UNSAFE_VERB ("FishAbout",       display_about_dialog),
 
         BONOBO_UI_VERB_END
 };
 
+
+
 static gboolean
 fish_applet_fill (PanelApplet *applet)
 {
@@ -1135,7 +1314,7 @@ fish_applet_fill (PanelApplet *applet)
 			  G_CALLBACK (applet_change_size),
 			  fish);
 
-	panel_applet_add_preferences (applet, "/schemas/apps/fish-applet/prefs", NULL);
+	panel_applet_add_preferences (applet, "/schemas/apps/fish_applet/prefs", NULL);
 
 	gtk_widget_push_colormap (gdk_rgb_get_cmap ());
 
@@ -1154,14 +1333,21 @@ fish_applet_fill (PanelApplet *applet)
 	load_image_file (fish);
 
 	create_fish_widget (fish);
-
+	
 	set_wanda_day ();
 
 	gtk_container_add (GTK_CONTAINER (fish->applet), fish->frame);
 
+	set_tooltip (GTK_WIDGET (fish->applet));
+
+	gtk_signal_connect (GTK_OBJECT (fish->applet), "key_press_event",
+			    GTK_SIGNAL_FUNC (fish_keypress_cb), fish);
+
 	gtk_widget_show_all (GTK_WIDGET (fish->frame));
 
 	gtk_widget_show (GTK_WIDGET (fish->applet));
+	
+	setup_size (fish);
 
 	panel_applet_setup_menu_from_file (PANEL_APPLET (fish->applet),
 				 NULL,
