@@ -93,7 +93,6 @@ int on_toplevel_key_press_event (GtkWidget *widget, GdkEventKey *key);
 /* some local prototypes */
 static gchar * add_file_to_path (const gchar *path);
 
-
 /* helper functions */
 /* This code is copied from gdk-pixbuf-HEAD.  It does no memory management and
  * is very hard-coded.  Please do not use it anywhere else. */
@@ -984,8 +983,8 @@ got_signal (int sig)
 	cleanup_temporary ();
 	
 	/* whack thyself */
-	signal (sig, SIG_DFL);
-	kill (getpid (), sig);
+	signal (sig, SIG_DFL); 
+	kill (getpid (), sig); 
 }
 
 static void
@@ -1004,7 +1003,6 @@ drag_begin (GtkWidget *widget, GdkDragContext *context)
 	start_temporary ();
 }
 
-
 /* main */
 int
 main (int argc, char *argv[])
@@ -1016,9 +1014,12 @@ main (int argc, char *argv[])
 	struct stat s;
 	gchar *file;
 	gboolean window = FALSE;
-	gint width, height;
+	gint width, height; 
+	guint delay = 0;
+
 	struct poptOption opts[] = {
 		{"window", '\0', POPT_ARG_NONE, &window, 0, N_("Grab a window instead of the entire screen"), NULL},
+		{"delay", '\0', POPT_ARG_INT, &delay, 0, N_("Take screenshot after specified delay [in seconds]"), NULL},
 		{NULL, '\0', 0, NULL, 0, NULL, NULL}
 	};
 
@@ -1036,7 +1037,11 @@ main (int argc, char *argv[])
 	glade_gnome_init();
 	client = gnome_master_client ();
 	gnome_client_set_restart_style (client, GNOME_RESTART_NEVER);
-
+	
+	if (delay > 0) {
+		sleep (delay);	
+	}
+	
 	if (window)
 		take_window_shot ();
 	else
@@ -1057,7 +1062,7 @@ main (int argc, char *argv[])
 			 GTK_MESSAGE_ERROR,
 			 GTK_BUTTONS_OK,
 			 _("Glade file for the screenshot program is missing.\n"
-			   "Please check your installation of gnome-core"));
+			   "Please check your installation of gnome-panel"));
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		exit (1);
 	}
@@ -1103,17 +1108,17 @@ main (int argc, char *argv[])
 			      FALSE);
 	if (window)
 		gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
+
 	home_dir = g_get_home_dir ();
 	web_dir = g_strconcat (home_dir, G_DIR_SEPARATOR_S,
 			       "public_html", NULL);
 
 	gconf_client = gconf_client_get_default ();
-	if (gconf_client_get_bool (gconf_client, "/apps/nautilus/preferences/desktop_is_home_dir", NULL)) {
-		desktop_dir = home_dir;
-	} else {
+	if (gconf_client_get_bool (gconf_client, "/apps/nautilus/preferences/desktop_is_home_dir", NULL))
+		desktop_dir = g_strdup (home_dir);
+	else
 		desktop_dir = g_strconcat (home_dir, G_DIR_SEPARATOR_S,
 					   ".gnome-desktop", NULL);
-	}
 	g_object_unref (gconf_client);
 	
 	file = add_file_to_path (home_dir);
