@@ -413,10 +413,13 @@ panel_applet_menu_position (GtkMenu  *menu,
 			    gpointer  data)
 {
 	GtkWidget *w = data;
+	GtkRequisition requisition;
 	gint wx, wy;
 
 	g_return_if_fail (w != NULL);
 
+	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
+
 	gdk_window_get_origin (w->window, &wx, &wy);
 	/*
          * Make sure that the popup position is in the panel
@@ -427,11 +430,17 @@ panel_applet_menu_position (GtkMenu  *menu,
 	else if (*x > wx + w->allocation.width)
 		*x = wx + w->allocation.width;
 
+	if (*x + requisition.width > gdk_screen_width())
+		*x = gdk_screen_width() - requisition.width;
+
 	if (*y < wy)
 		*y = wy;
 	 else if (*y > wy + w->allocation.height)
 		*y = wy + w->allocation.height;
 
+	if (*y + requisition.height > gdk_screen_height())
+		*y = gdk_screen_height() - requisition.height;
+
 	*push_in = TRUE;
 }
 
@@ -441,7 +450,9 @@ panel_applet_button_press (GtkWidget      *widget,
 {
 	PanelApplet *applet = PANEL_APPLET (widget);
 
-	if (event->button == 3) {
+	if (event->button == 1)
+		return TRUE;
+	else if (event->button == 3) {
 		bonobo_control_do_popup (applet->priv->control, 
 					 event->button,
 					 event->time);
@@ -452,14 +463,21 @@ panel_applet_button_press (GtkWidget      *widget,
 }
 
 gboolean
-_panel_applet_popup_menu (PanelApplet *applet)
+_panel_applet_popup_menu (PanelApplet *applet,
+			  guint button,
+			  guint32 time)			  
 {
 	bonobo_control_do_popup_full (applet->priv->control, NULL, NULL,
 				      panel_applet_menu_position,
-				      GTK_WIDGET (applet), 3,
-				      GDK_CURRENT_TIME);
+				      GTK_WIDGET (applet), button,
+				      time);
 	return TRUE;
-	
+}
+
+gboolean
+panel_applet_popup_menu (PanelApplet *applet)
+{
+	return _panel_applet_popup_menu (applet, 3, GDK_CURRENT_TIME);
 }
 
 static gboolean
@@ -1176,7 +1194,7 @@ panel_applet_setup (PanelApplet *applet)
 				     BONOBO_OBJECT (priv->item_handler));
 
 	g_signal_connect (applet, "popup_menu",
-			  G_CALLBACK (_panel_applet_popup_menu), NULL);
+			  G_CALLBACK (panel_applet_popup_menu), NULL);
 }
 
 /**
